import { Component, OnInit } from '@angular/core';
import { Router, NavigationEnd, ActivatedRoute } from '../../node_modules/@angular/router';
import { filter, map } from 'rxjs/operators';
import { AppRoutes } from './routes';
import { NavigationHistoryService } from './core/services';
import { Title } from '@angular/platform-browser';
import { Angulartics2Piwik } from 'angulartics2/piwik';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
})
export class AppComponent implements OnInit {

  constructor(
    private _router: Router,
    private _navigationHistoryService: NavigationHistoryService,
    private _activatedRoute: ActivatedRoute,
    private _titleService: Title,
    private _angulartics2Piwik: Angulartics2Piwik,
  ) {
    this._angulartics2Piwik.startTracking();
  }

  ngOnInit() {
    this._router.events.pipe(
      filter(e => e instanceof NavigationEnd),
    ).subscribe((e) => {
      this._navigationHistoryService.add(e['urlAfterRedirects']);
    });

    // Change the title using the title property passed in the data of each route
    this._router.events.pipe(
      filter(event => event instanceof NavigationEnd),
      map(() => this._activatedRoute.snapshot),
      map((route) => {
        const lang = window.navigator.language.includes('fr') ? 'fr' : 'en';
        let r = route;
        const titles = [AppRoutes.root.title[lang]];
        while (r.firstChild) {
          r = r.firstChild;
          if (r.outlet === 'primary' && r.data.title != null) {
            titles.unshift(r.data.title[lang]);
          }
        }
        return titles;
      }),
    ).subscribe((titles) => {
      const title = titles.join(' • ');
      this._titleService.setTitle(title);
    });
  }

  // Return true if origin url and destination url are both in datasetDetail
  isRoutingInsideDatasetDetail(origin, destination) {
    let res = false;
    if (origin.search(`${AppRoutes.datasets.uri}/`) !== -1 &&
      destination.search(`${AppRoutes.datasets.uri}/`) !== -1) {
      res = true;
    }
    return res;
  }
}
