import { Component, OnInit } from '@angular/core';
import { NavigationEnd, Router } from '@angular/router';
import { Angulartics2Piwik } from 'angulartics2/piwik';
import { AppRoutes } from '../../../../routes';
import { UserService } from '../../../../user/services';
import { AppStateService } from '../../../services';

@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.scss'],
})
export class HeaderComponent implements OnInit {

  isBurgerActive = false;
  isHomePage = true;
  userDropdownIsOpened = false;

  AppRoutes = AppRoutes;

  constructor(
    private _router: Router,
    private _userService: UserService,
    private _appStateService: AppStateService,
    private _angulartics2Piwik: Angulartics2Piwik,
  ) { }

  ngOnInit() {
    this._router.events.subscribe((event) => {

      if (event instanceof NavigationEnd) {
        this.isHomePage = (event.url === `/${AppRoutes.home.uri}` || event.url === '/') ? true : false;
      }
    });

    this._appStateService.menuStateChanged$.subscribe((state) => {
      this.isBurgerActive = state;
    });
  }

  toggleBurger() {
    this.isBurgerActive = !this.isBurgerActive;
    this._appStateService.changeMenuState(this.isBurgerActive);
    this._angulartics2Piwik.eventTrack('BurgerMenu', { category: 'Navigation', label: 'BurgerManu', value: 1 });
  }

  toggleUserDropdown() {
    this.userDropdownIsOpened = !this.userDropdownIsOpened;
  }

  toogleFeedback(event) {
    event.stopPropagation(); // Avoid to trigger clickOutside directive
    this._appStateService.changeFeedbackState(!this._appStateService.feedbackOpened);
  }

  signOut() {
    this._userService.resetAuth();
    this.closeUserDropdown();
    this._router.navigate(['/', AppRoutes.home.uri]);
  }

  closeUserDropdown() {
    this.userDropdownIsOpened = false;
  }

  get userIsSignedIn() {
    return this._userService.userIsSignedIn;
  }

  get username() {
    return this._userService.user.firstName;
  }
}
