import { Component, OnDestroy, OnInit } from '@angular/core';
import { NavigationEnd, Router } from '@angular/router';
import { Subscription } from 'rxjs';
import { AppRoutes } from '../../../routes';
import { AppStateService } from '../../services';

@Component({
  selector: 'app-main',
  templateUrl: './main.component.html',
  styleUrls: ['./main.component.scss'],
})
export class MainComponent implements OnInit, OnDestroy {

  sidebarOpened = false;
  isHomePage = false;
  isDatasetDetailTableMapPage = false;
  subscriptions: Subscription[] = [];

  constructor(
    private _router: Router,
    private _appStateService: AppStateService,
  ) { }

  ngOnInit() {
    // Check if homepage. This is used to color the background (to make side-menu transition possible)
    this._router.events.subscribe((event) => {
      if (event instanceof NavigationEnd) {
        this.isHomePage = (event.url === `/${AppRoutes.home.uri}` || (event.url === '/')) ? true : false;
        const regex = new RegExp(`${AppRoutes.datasets.uri}\/.+\/${AppRoutes.data.uri}`);
        this.isDatasetDetailTableMapPage = event.url.match(regex) !== null ? true : false;
      }
    });

    this.subscriptions.push(
      this._appStateService.menuStateChanged$.subscribe((state) => {
        this.sidebarOpened = state;
      }),
    );
  }

  ngOnDestroy() {
    this.subscriptions.forEach((sub) => {
      sub.unsubscribe();
    });
  }

  closeSideMenu() {
    this.sidebarOpened = false;
    this._appStateService.changeMenuState(false);

  }

  get feedbackIsOpened() {
    return this._appStateService.feedbackOpened;
  }
}
