import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { geosource } from '../../../../../../i18n/traductions';
import { IMetadataLink, linkFormats } from '../../../../../shared/models';
import { Format, Resource } from '../../../../models';

@Component({
  selector: 'app-resource-download-item',
  templateUrl: './resource-download-item.component.html',
  styleUrls: ['./resource-download-item.component.scss']
})
export class ResourceDownloadItemComponent implements OnInit {

  linkFormats = linkFormats;

  @Input() format?: Format;
  @Input() resource: Resource;
  @Input() link: IMetadataLink;
  @Input() isQueryable: boolean;
  @Output() saveEvent = new EventEmitter();
  @Output() abortEvent = new EventEmitter();

  abortMessage: string = geosource.downloads.abort;
  downloadMessage: string = geosource.downloads.download;

  labelLayer = {
    WFS: 'typename',
    WMS: 'layers',
    WCS: 'identifiers',
  };

  labelFormat = {
    WFS: 'outputFormat',
    WMS: 'FORMAT',
    WCS: 'FORMAT',
  };

  constructor() { }

  ngOnInit() {
  }

  getFormatName() {
    return this.format ? this.format.name : this.link.formats[0];
  }

  getQueryableUrl() {
    let queryableUrl = '';
    if (this.isQueryable) {

      queryableUrl = this.resource.metadataLink.url;
      let layer = '';
      let outputFormat = '';
      if (this.resource.isStandard) {
        layer = `&${this.labelLayer[this.resource.type]}=${this.resource.metadataLink.name}`;
        outputFormat = `&${this.labelFormat[this.resource.type]}=` +
          `${this.format.mapServerType}`;
        const projectionAndBbox = this.getProjectionAndBbox(this.format);

        const baseParameters = this.resource.parametersUrl ? `?${this.resource.parametersUrl}` : '';
        queryableUrl += baseParameters +
          layer +
          outputFormat +
          projectionAndBbox.projection +
          projectionAndBbox.bboxRequest;
      } else if (this.resource.type === linkFormats.ws) {
        if (this.format.name === 'JSON') {
          queryableUrl += `/${this.resource.metadataLink.name}/all.json`;
        } else {
          queryableUrl += `/${this.resource.metadataLink.name}.shp?srsname=EPSG:3946`;
        }
      }
    } else {
      queryableUrl = this.link.url;
    }
    return queryableUrl;
  }

  getFileName() {
    return this.resource ? `${this.resource.metadataLink.name}.${this.format.fileExtension}` : this.link.name;
  }

  getProjectionAndBbox(format: Format) {
    let projection = '';
    let bboxRequest = '';
    if (this.resource.type === linkFormats.wfs) {
      projection = '&SRSNAME=EPSG:3946';
    } else if (this.resource.type === linkFormats.wms || this.resource.type === linkFormats.wcs) {
      projection = '&CRS=EPSG:3946';
      const bbox = this.resource.metadataLink.bbox_by_projection['EPSG:3946'];
      bboxRequest = `&BBOX=${bbox.minx},` +
        `${bbox.miny},` +
        `${bbox.maxx},` +
        `${bbox.maxy}`;
    } else if (this.resource.type === linkFormats.ws) {
      if (format.name !== 'JSON') {
        projection = 'srsname=EPSG:3946';
      }
    }

    return {
      projection,
      bboxRequest,
    };
  }

}
