import { async, ComponentFixture, TestBed } from '@angular/core/testing';
import { of, Observable } from 'rxjs';
import { DatasetInfoComponent } from './dataset-info.component';
import { DatasetDetailService } from '../../../services';
import { Metadata } from '../../../models';
import { RouterTestingModule } from '@angular/router/testing';
import { DebugElement } from '@angular/core';
import { MockComponent } from 'ng2-mock-component';
import { By } from '@angular/platform-browser';
import { EditorialisationService } from '../../../../editorialisation/services';

const metadata = {
  title: 'Metadata Title',
  abstract: 'abstract',
  abstractTroncated: '',
  providers: [],
  keyword: ['riri', 'fifi', 'loulou'],
  getCategories: () => { return ['space_exploration', 'tintin_sur_la_lune']; },
  dataset_id: '1',
  link: [],
  contact: {
    role: '',
    organisationName: '',
    address: '',
    email: '',
    appliesTo: '',
  },
  publicationDate: '',
  category: '',
  highlight: '',
  legalConstraints: [],
  rights: [],
  total_documents: 10,
  lineage: '',
  type: '',
  denominator: 2,
  resolution: '',
  topic_category: [],
  geonet: {
    notify: '',
    schema: '',
    createDate: '',
    view: '',
    id: '',
    featured: '',
    download: '',
    changeDate: '',
    dynamic: '',
    '@xmlns:geonet': '',
    source: '',
    uuid: '',
    selected: '',
    isPublishedToAll: '',
  },
  categories: [],
  popularity: '',
  max_east: 4,
  max_south: 4,
  max_north: 4,
  updateFrequency: '',
  max_west: 4,
  crs: '',
  image: {
    url: '',
    type: '',
  },

} as Metadata;

describe('DatasetInfoComponent', () => {
  describe('Template', () => {
    let component: DatasetInfoComponent;
    let fixture: ComponentFixture<DatasetInfoComponent>;
    let de: DebugElement;

    beforeEach(async(() => {
      TestBed.configureTestingModule({
        declarations: [
          DatasetInfoComponent,
          MockComponent({ selector: 'app-dataset-description', inputs: ['description'] }),
        ],
        imports: [
          RouterTestingModule,
        ],
        providers: [
          {
            provide: DatasetDetailService,
            useValue: {
              get datasetMetadata() {
                return metadata;
              },
              get dataset$(): Observable<void> {
                return of();
              },
              get datasetChildren() {
                return { data: [] };
              },
            },
          },
          {
            provide: EditorialisationService,
            useValue: {
              getPostsByTextSearch(toto) {
                return of([]) ;
              },
            },
          },
        ],
      }).compileComponents();
    }));

    beforeEach(() => {
      fixture = TestBed.createComponent(DatasetInfoComponent);
      component = fixture.componentInstance;
      component.descriptions = [];
      de = fixture.debugElement;
      fixture.detectChanges();
    });

    it('should create component', () => {
      expect(component).toBeTruthy(); // test la config test bed
    });

    it('should render metadata information', () => {
      // Given
      const metadataAbstractElement = de.query(By.css('.metadata-abstract')).nativeElement;

      // Then
      expect(metadataAbstractElement.innerText).toBe(metadata.abstract);
    });
  });

  describe('Component.ts', () => {
    beforeEach(() => {

    });

    it('ngOnInit()', () => {
      // Given
      const serviceMock = {
        get datasetMetadata() {
          return metadata;
        },
        get dataset$(): Observable<void> {
          return of();
        },
        get datasetChildren() {
          return { data: [] };
        },
      } as DatasetDetailService;
      const serviceEditoMock = {
        getPostsByTextSearch(toto) {
          return of([]) ;
        },
      } as EditorialisationService;
      const datasetSpy = spyOnProperty(serviceMock, 'dataset$').and.callThrough();

      const component = new DatasetInfoComponent(serviceMock, serviceEditoMock);

      // When
      component.ngOnInit();

      // Then
      expect(datasetSpy).toHaveBeenCalled();

    });
  });

});
