
export interface IESCMSResponse {
  'content-fr': IGhostContentResponse;
  type: string;
  uuid: string;
  slug: string;
  relations: IESCMSRelations;
}

export interface IESCMSRelations {
  datasets: {
    slugs: string[];
  };
  posts: {
    slugs: string[];
  };
  pages: {
    slugs: string[];
  };
}

export interface IGhostContentResponse {
  id: string;
  // iso8601 format
  published_at: string;
  slug: string;
  updated_at: string;
  title: string;
  status: string;
  html: string;
  excerpt: string;
  feature_image: string;
  primary_author: IGhostAuthor;
  featured: boolean;
  tags: IGhostTag[];
  highlight?: IHighlight[];
}

export interface IHighlight {
  label: string;
  value: string;
}

interface IGhostAuthor {
  id: number;
  name: string;
}

interface IGhostTag {
  id: number;
  name: string;
}

export class CMSRelations {
  datasets: {
    slugs: string[];
  };
  posts: {
    slugs: string[];
  };
  pages: {
    slugs: string[];
  };

  constructor(data: IESCMSRelations) {
    this.pages = data && data.pages && data.pages.slugs ? data.pages : { slugs: [] };
    this.posts = data && data.posts && data.posts.slugs ? data.posts : { slugs: [] };
    this.datasets = data && data.datasets && data.datasets.slugs ? data.datasets : { slugs: [] };
  }
}

export class CMSContent {
  content: GhostContentResponse;
  type: string;
  uuid: string;
  slug: string;
  relations: CMSRelations;

  constructor(data: IESCMSResponse) {
    this.content = (data) ? new GhostContentResponse(data['content-fr']) : null;
    this.type = (data.type) ? data.type : '';
    this.uuid = (data.uuid) ? data.uuid : '';
    this.slug = (data['content-fr'].slug) ? data['content-fr'].slug : '';
    this.relations = new CMSRelations(data.relations);
  }
}

export class GhostContentResponse {
  id: string;
  title: string;
  status: string;
  html: string;
  excerpt: string;
  featureImage: string;
  author: IGhostAuthor;
  tags: IGhostTag[];
  category: IGhostTag;
  featured: boolean;
  publicationDate: number;
  modificationDate: number;
  highlight?: string[];
  titleHighlight?: string;

  constructor(data: IGhostContentResponse) {
    this.id = data.id;
    this.status = (data.status != null) ? data.status : '';
    this.title = (data.title != null) ? data.title : '';
    this.html = (data.html != null) ? data.html : '';
    this.excerpt = (data.excerpt != null) ? data.excerpt : '';
    this.featureImage = data.feature_image;
    this.publicationDate = (data.published_at != null) ? Date.parse(data.published_at) : 0;
    this.modificationDate = (data.updated_at != null) ? Date.parse(data.updated_at) : 0;

    // Ghost tags are separated in 2 ways:
    // - tags starting with '#' are tags (one or many per article)
    // - the tag not starting with '#' is the categorie (only one per article)
    this.tags = [];
    this.category = null;
    if (data.tags) {

    }
    data.tags.forEach((tag) => {
      tag.name.startsWith('#') ? this.tags.push(tag) : this.category = tag;

    });

    this.author = (data.primary_author) ? data.primary_author : null;
    this.highlight = [];
  }
}

export interface ICMSMedia {
  id: number;
  guid: {
    rendered: string;
  };
}

export interface ICMSCategory {
  id: number;
  name: string;
}
