export interface IOrganization {
  id: number;
  name: string;
  description: string;
  elasticSearchName?: string;
  links: ILink[];
  logo: string;
}

interface ILink {
  id: number;
  name: string;
  url: string;
  organizationId: number;
}

class Link {
  id: number;
  name: string;
  url: string;
  urlLabel: string;
  organizationId: number;

  constructor(iLink: ILink) {
    this.id = iLink.id;
    this.name = iLink.name;
    this.url = iLink.url;
    // For the label url to display: remove http|https from url
    this.urlLabel = iLink.url.replace(/^(https?:|)\/\//, '');
    this.urlLabel = this.urlLabel.replace(/\/$/, '');
    this.organizationId = iLink.organizationId;
  }
}

export class Organization implements IOrganization {
  id: number;
  name: string;
  description: string;
  links: ILink[];
  logo: string;
  elasticSearchName?: string;
  datasetsCount?: number;
  servicesCount?: number;

  constructor(organization: IOrganization) {
    this.id = organization.id;
    this.name = organization.name;
    this.description = organization.description;
    this.links = [];
    organization.links.forEach((link) => {
      this.links.push(new Link(link));
    });
    this.logo = organization.logo;
    this.elasticSearchName = organization.elasticSearchName;
    this.datasetsCount = 0;
    this.servicesCount = 0;
  }
}
