import { Component, EventEmitter, OnInit, Output } from '@angular/core';
import { DomSanitizer, SafeHtml } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { CookieService } from 'ngx-cookie-service';
import { environment } from '../../../../environments/environment';
import { CMSContent } from '../../../editorialisation/models';
import { EditorialisationService } from '../../../editorialisation/services';
import { AppRoutes } from '../../../routes';
import { UserService } from '../../../user/services';

@Component({
  selector: 'app-cgu-modal',
  templateUrl: './cgu-modal.component.html',
  styleUrls: ['./cgu-modal.component.scss']
})
export class CguModalComponent implements OnInit {

  AppRoutes = AppRoutes;
  cguModalIsOpened: boolean;
  safePageContent: SafeHtml;
  isCguScrolledEntirely = false;

  @Output() cguAccepted = new EventEmitter();

  constructor(
    private _editorialisationService: EditorialisationService,
    private sanitizer: DomSanitizer,
    private _router: Router,
    private _route: ActivatedRoute,
    private _cookieService: CookieService,
    private _userService: UserService,
  ) { }

  ngOnInit() {
    this.cguModalIsOpened = (this._cookieService.get(environment.cguAcceptedCookieName) !== 'true')
      && !this._userService.userIsSignedIn;

    if (this.cguModalIsOpened) {
      this._editorialisationService.getPage(environment.cmsStaticPages[AppRoutes.cgu.uri]).subscribe((page) => {
        if (!(page instanceof CMSContent)) {
        } else {
          this.safePageContent = this.sanitizer.bypassSecurityTrustHtml(page.content.html);
        }
      });
    }
  }

  closeCguModal() {
    this.cguModalIsOpened = false;
    this._router.navigate([`../${AppRoutes.data.uri}`], { relativeTo: this._route });
  }

  acceptCgu() {
    // The div containing the cgu must be scrolled to the bottom in order to be able to accept the cgu
    if (this.isCguScrolledEntirely) {
      const expDate = new Date(new Date().setFullYear(new Date().getFullYear() + 1));
      this._cookieService.set(
        environment.cguAcceptedCookieName, // Key
        'true', // Value
        expDate,
        '/',
      );
      this.cguModalIsOpened = false;
      this.cguAccepted.next();
    }
  }

  modalBodyScrolled(event) {
    const target = event.target || event.srcElement;
    if (target.scrollTop + target.offsetHeight >= target.scrollHeight) {
      this.isCguScrolledEntirely = true;
    }
  }
}
