import { Component, HostListener, Input, OnInit } from '@angular/core';
import { geosource } from '../../../../i18n/traductions';

@Component({
  selector: 'app-download-button',
  templateUrl: './download-button.component.html',
  styleUrls: ['./download-button.component.scss'],
})
export class DownloadButtonComponent implements OnInit {

  loading: boolean = false;
  @Input() url: string;
  @Input() fileName: string;

  // Event fired when one tab is closed or refreshed. We abort the on-going request
  @HostListener('window:beforeunload')
  abortCurrentRequest() {
    this.controller.abort();
  }

  // Variable to manage the icon display for each resource. Possible values:
  // - 'loading': display the spinner
  // - 'cancel': display the cross to abort the on-going request
  // -  null: display the download icon
  signal: AbortSignal;
  controller: AbortController;
  timeOut: number;

  abortMessage: string = geosource.downloads.abort;
  downloadMessage: string = geosource.downloads.download;

  constructor() { }

  ngOnInit() {
    this.controller = new AbortController();
    this.signal = this.controller.signal;
  }

  download() {
    this.loading = true;

    fetch(this.url, { signal: this.signal }).then((response) => {
      // To allow the download, get the repsons as a blob
      return response.blob();
    })
      .then((blob) => {
        // Create a temporary link and click on it to launch the blob download
        const url = window.URL.createObjectURL(blob);
        const a = document.createElement('a');
        a.href = url;
        a.rel = 'noopener';
        a.download = this.fileName;
        document.body.appendChild(a); // append the element to the dom -> otherwise it will not work in firefox
        a.click();
        a.remove();
        this.loading = false;
        clearTimeout(this.timeOut); // If the download is under 2000ms, we clear the ongoing timeout
      })
      .catch((err) => {
        console.log('The fetch could not succeeded', err.message);
      });
  }

  abortDownload() {

    this.controller.abort();
    this.loading = false;

    // Create a new for a new DOM Request
    this.controller = new AbortController();
    this.signal = this.controller.signal;
  }

  ngOnDestroy() {
    this.controller.abort();
  }
}
