import { HttpInterceptor, HttpRequest, HttpHandler, HttpEvent } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { CookieService } from 'ngx-cookie-service';

@Injectable()
export class XSRFTokenInterceptor implements HttpInterceptor {

  constructor(private cookieService: CookieService) { }

  intercept(
    req: HttpRequest<any>,
    next: HttpHandler,
  ): Observable<HttpEvent<any>> {
    const xsrfToken = this.cookieService.get('XSRF-TOKEN');
    let request = req;

    if (xsrfToken) {
      request = req.clone({
        headers: req.headers.set('x-xsrf-token', xsrfToken),
      });
    }
    return next.handle(request);
  }
}
